from functools import wraps
from os import path
from os.path import join, basename

import requests
from requests import HTTPError
from requests_oauthlib import OAuth2Session

from django_adobesign.exceptions import get_adobe_exception

ADOBE_OAUTH_TOKEN_URL = 'https://api.echosign.com/oauth/token'
ADOBE_OAUTH_REFRESH_TOKEN_URL = 'https://api.echosign.com/oauth/refresh'


class AdobeSignOAuthSession(object):
    def __init__(self, application_id, redirect_uri, account_type, state=None):
        self.application_id = application_id
        self.oauth_session = OAuth2Session(
            client_id=self.application_id,
            redirect_uri=redirect_uri,
            scope=self.get_scopes(account_type),
            state=state)

    def get_authorization_url(self, root_url):
        url = join(root_url, 'public/oauth')
        authorization_url, state = self.oauth_session.authorization_url(url)
        return authorization_url

    @staticmethod
    def get_scopes(account_type):
        return [scope.format(account_type) for scope in (
            'user_login:{}',
            'agreement_send:{}',
            'agreement_read:{}',
            'agreement_write:{}')]

    def create_token(self, code, application_secret):
        response = self.oauth_session.fetch_token(
            ADOBE_OAUTH_TOKEN_URL,
            code=code,
            client_secret=application_secret,
            authorization_response='/')

        return response

    @staticmethod
    def refresh_token(refresh_token, application_id, application_secret):
        oauth_session = OAuth2Session(client_id=application_id)
        response = oauth_session.refresh_token(
            ADOBE_OAUTH_REFRESH_TOKEN_URL,
            refresh_token=refresh_token,
            client_id=application_id,
            client_secret=application_secret,
            authorization_response="/")

        return response


def handle_adobe_exception(function):
    @wraps(function)
    def wrapper(*arg, **kwargs):
        try:
            return function(*arg, **kwargs)
        except (requests.exceptions.RequestException, HTTPError) as e:
            raise get_adobe_exception(e)

    return wrapper


class AdobeSignClient(object):
    '''
    AdobeSign client use v6 api.
    See https://secure.na1.echosign.com/public/docs/restapi/v6
    '''

    def __init__(self, root_url, access_token, api_user=None,
                 on_behalf_of_user=None):
        self.root_url = root_url.strip('/')
        self.access_token = access_token
        self.on_behalf_of_user = on_behalf_of_user
        self.api_user = api_user

    def build_url(self, urlpath):
        return path.join(self.root_url, 'api/rest/v6', urlpath)

    def get_headers(self):
        header = {'Authorization': 'Bearer {}'.format(self.access_token)}
        if self.api_user:
            header['x-api-user'] = self.api_user
        if self.on_behalf_of_user:
            header['x-on-behalf-of-user'] = self.on_behalf_of_user
        return header

    @handle_adobe_exception
    def upload_document(self, document):
        """
            Upload a document and get a transient document id
        """
        url = self.build_url(urlpath='transientDocuments')
        data = {
            'File-Name': basename(document.name),
            'Mime-Type': 'application/pdf'
        }

        response = requests.post(url,
                                 headers=self.get_headers(),
                                 files={'File': document.bytes},
                                 data=data)
        response.raise_for_status()
        return response.json()

    def jsonify_participant(self, name, email, order):
        return {
            'name': name,
            'memberInfos': [
                {'email': email}],
            'order': order,
            'role': 'SIGNER'
        }

    @handle_adobe_exception
    def post_agreement(self, transient_document_id, name, participants,
                       post_sign_redirect_url, post_sign_redirect_delay,
                       send_mail, state='IN_PROCESS', **extra_data):
        '''
        Create signature with only one document

        https://secure.na1.adobesign.com/public/docs/restapi/v6#!/agreements/createAgreement

        This is a primary endpoint which is used to create a new agreement.
        An agreement can be created using transientDocument, libraryDocument
        or a URL. You can create an agreement in one of the 3 mentioned
        states:
        a) DRAFT - to incrementally build the agreement before
        sending out,
        b) AUTHORING - to add/edit form fields in the agreement,
        c) IN_PROCESS - to immediately send the agreement.

        :param post_sign_redirect_delay: The delay (in seconds) before the user
        is taken to the success page. If this value is greater than 0, the user
        will first see the standard Adobe Sign success message, and then after
        a delay will be redirected to your success page,
        :param post_sign_redirect_url: A publicly accessible url to which the
        user will be sent after successfully completing the signing process
        :param document: file wrapper object
        :param name: name of document
        :param participants:  A list of one or more participant set.
        A participant set may have one or more participant.
        If any member of the participant set takes the action that has been
        assigned to the set(Sign/Approve/Acknowledge etc ), the action is
        considered as the action taken by whole participation set.
        For regular (non-MegaSign) documents, there is no limit on the number
        of electronic signatures in a single document. Written signatures are
        limited to four per document,
        :param extra_data: extra data to pass (see adobesign documentation)
        '''

        # Send doc for signature
        url = self.build_url(urlpath='agreements')

        data = {
            'fileInfos': [{
                'transientDocumentId': transient_document_id
            }],
            'name': name,
            'participantSetsInfo': participants,
            'signatureType': 'ESIGN',
            'state': state
        }
        if post_sign_redirect_url:
            data['postSignOption'] = {
                'redirectDelay': post_sign_redirect_delay,
                'redirectUrl': post_sign_redirect_url
            }

        if not send_mail:
            data['emailOption'] = {
                'sendOptions': {
                    'completionEmails': 'NONE',
                    'inFlightEmails': 'NONE',
                    'initEmails': 'NONE'
                }
            }

        data.update(extra_data)
        response = requests.post(url,
                                 headers=self.get_headers(),
                                 json=data)
        response.raise_for_status()
        return response.json()

    @handle_adobe_exception
    def get_agreements(self, page_size, cursor=None, **extra_params):
        """
        Return the list of agreements with pagination
        """
        path_url = 'agreements'
        params = {'pageSize': page_size}
        if cursor:
            params['cursor'] = cursor
        params.update(extra_params)
        url = self.build_url(path_url)
        response = requests.get(url, headers=self.get_headers(),
                                params=params)
        response.raise_for_status()
        return response.json()

    @handle_adobe_exception
    def get_members(self, agreement_id, include_next_participant_set):
        """
        Return members of a given agreement id
        :param include_next_participant_set: add to the response the set of
        next participants
        """
        url = self.build_url('agreements/{}/members'.format(agreement_id))
        params = {
            'includeNextParticipantSet': include_next_participant_set}
        response = requests.get(url,
                                params=params,
                                headers=self.get_headers())
        response.raise_for_status()
        return response.json()

    @handle_adobe_exception
    def get_signing_url(self, agreement_id):
        """
        Return the next signing url for the agreement
        corresponding to the agreement_id.
        """
        url = self.build_url('agreements/{}/signingUrls'.format(agreement_id))
        response = requests.get(url, headers=self.get_headers())
        response.raise_for_status()
        return response.json()

    @handle_adobe_exception
    def get_signer(self, agreement_id, signer_id):
        """
        Return the signer with the given signer_id who belongs to the agreement
        corresponding to the agreement_id.
        """
        url = self.build_url('agreements/{}/members/participantSets/{}'
                             .format(agreement_id, signer_id))
        response = requests.get(url, headers=self.get_headers())
        response.raise_for_status()
        return response.json()

    @handle_adobe_exception
    def get_documents(self, agreement_id, **extra_data):
        """
        Return all document ids for a given agreement id
        """
        url = self.build_url('agreements/{}/documents'.format(agreement_id))
        response = requests.get(url, headers=self.get_headers(),
                                data=extra_data)
        response.raise_for_status()
        return response.json()

    @handle_adobe_exception
    def get_document(self, agreement_id, document_id):
        """
        Download a document
        """
        url = self.build_url('agreements/{}/documents/{}'
                             .format(agreement_id, document_id))
        response = requests.get(url, headers=self.get_headers())
        response.raise_for_status()
        return response.content

    def rebuild_with_token(self, access_token):
        return AdobeSignClient(self.root_url, access_token, self.api_user,
                               self.on_behalf_of_user)



import pytest
from adobesign.models import Signer, Signature, SignatureType
from django.core.files import File
from django.db.models import FileField

from django_adobesign.backend import AdobeSignBackend
from django_adobesign.client import AdobeSignClient
from django_adobesign.exceptions import AdobeSignNoMoreSignerException


class AdobeSignBackendTest(AdobeSignBackend):

    def update_signer_status(self, signer, status):
        pass


@pytest.fixture()
def adobe_sign_backend():
    adobe_sign_client = AdobeSignClient(root_url='http://fake',
                                        access_token='ThisIsAToken')
    return AdobeSignBackendTest(adobe_sign_client)


@pytest.fixture()
def minimal_signature(mocker):
    signature_type = SignatureType()
    signature_type.save()
    signature = Signature(signature_type=signature_type)
    signature.document = mocker.Mock(FileField)
    signature.document.file = mocker.Mock(File)
    signature.document.file.name = '/tmp/uploaded_file.pdf'
    signature.document._committed = True
    signature.save()
    return signature


@pytest.mark.django_db
def test_get_adobesign_participants_in_right_order(minimal_signature,
                                                   adobe_sign_backend):
    signer1 = Signer(full_name='Poney poney', email='poney@plop.com',
                     signing_order=1)
    signer2 = Signer(full_name='Pouet pouet', email='pouet@plop.com',
                     signing_order=2)

    signer2.signature = minimal_signature
    signer1.signature = minimal_signature

    signer2.save()
    signer1.save()
    adobe_sign_signers = adobe_sign_backend.get_adobesign_participants(
        minimal_signature)

    assert len(adobe_sign_signers) == 2

    expected_signer1 = {
        'name': 'Poney poney',
        'memberInfos': [
            {'email': 'poney@plop.com'}],
        'order': 1,
        'role': 'SIGNER'
    }
    expected_signer2 = {
        'name': 'Pouet pouet',
        'memberInfos': [
            {'email': 'pouet@plop.com'}],
        'order': 2,
        'role': 'SIGNER'
    }
    assert adobe_sign_signers[0] == expected_signer1
    assert adobe_sign_signers[1] == expected_signer2


@pytest.mark.django_db
def test_create_signature(mocker, minimal_signature, adobe_sign_backend):
    mocker.patch.object(AdobeSignClient, 'upload_document',
                        return_value={'transientDocumentId': 'doc_id'})
    mocker.patch.object(AdobeSignClient, 'post_agreement',
                        return_value={'id': 'test_agreement_id'})
    mocker.patch.object(
        AdobeSignClient, 'get_members',
        return_value={
            'participantSets': [
                {'memberInfos': [{'email': 'pouet@plop.com'}],
                 'id': 'fooid',
                 'status': 'NOT_YET_VISIBLE',
                 'order': 2},
                {'memberInfos': [{'email': 'poney@plop.com'}],
                 'id': 'barid',
                 'status': 'CANCELLED',
                 'order': 1}]})

    # Signers
    signer1 = Signer(full_name='Poney poney', email='poney@plop.com',
                     signing_order=1)
    signer2 = Signer(full_name='Pouet pouet', email='pouet@plop.com',
                     signing_order=2)
    signer2.signature = minimal_signature
    signer1.signature = minimal_signature
    signer2.save()
    signer1.save()

    assert signer1.signature_backend_id == ""
    assert signer2.signature_backend_id == ""

    assert minimal_signature.signature_backend_id == ""

    adobe_sign_backend.create_signature(minimal_signature)

    minimal_signature.refresh_from_db()
    assert minimal_signature.signature_backend_id == 'test_agreement_id'

    signer1.refresh_from_db()
    signer2.refresh_from_db()
    assert signer1.signature_backend_id == "barid"
    assert signer2.signature_backend_id == "fooid"


def test_get_next_signer_urls(mocker, adobe_sign_backend):
    mocker.patch.object(AdobeSignClient, 'get_signing_url',
                        side_effect=AdobeSignNoMoreSignerException('m', 'c'))
    with pytest.raises(AdobeSignNoMoreSignerException):
        adobe_sign_backend.get_next_signer_urls('12')


def test_get_next_signer_url(mocker, adobe_sign_backend):
    data = {'signingUrlSetInfos': [{
        'signingUrls': [{
            'email': 'pouet@truc.com',
            'esignUrl': 'url'},
            {'other': 'potential signer'}
        ]}
    ]}
    mocker.patch.object(AdobeSignClient, 'get_signing_url',
                        return_value=data)
    mail, url = adobe_sign_backend.get_next_signer_url('12')
    assert (mail, url) == ('pouet@truc.com', 'url')


@pytest.mark.django_db
def test_is_last_signer(adobe_sign_backend, minimal_signature):
    signer1 = Signer(full_name='A', email='a@b.c', signing_order=1)
    signer2 = Signer(full_name='B', email='d@e.f', signing_order=2)
    signer2.signature = minimal_signature
    signer1.signature = minimal_signature
    signer2.save()
    signer1.save()

    assert not adobe_sign_backend.is_last_signer(signer1)
    assert adobe_sign_backend.is_last_signer(signer2)



import pytest
from requests import Response

from django_adobesign.client import AdobeSignOAuthSession, \
    ADOBE_OAUTH_TOKEN_URL, AdobeSignClient, ADOBE_OAUTH_REFRESH_TOKEN_URL
from django_adobesign.exceptions import AdobeSignException, \
    AdobeSignNoMoreSignerException, AdobeSignInvalidAccessTokenException, \
    AdobeSignInvalidUserException


@pytest.fixture()
def adobe_oauth_session():
    return AdobeSignOAuthSession(application_id='na',
                                 redirect_uri='na',
                                 account_type='na')


def test_oauth_get_authorization_url(adobe_oauth_session):
    expected_root_url = 'https://toto'
    authorization_url = adobe_oauth_session.get_authorization_url(
        expected_root_url)

    assert authorization_url.startswith('https://toto/public/oauth')


def test_oauth_get_scopes():
    scopes = AdobeSignOAuthSession.get_scopes(account_type='account')

    assert 'user_login:account' in scopes
    assert 'agreement_send:account' in scopes
    assert 'agreement_read:account' in scopes
    assert 'agreement_write:account' in scopes


def test_oauth_create(mocker, adobe_oauth_session):
    mocked_function = mocker.patch.object(adobe_oauth_session.oauth_session,
                                          'fetch_token')
    adobe_oauth_session.create_token('code_test', 'appli_secret_test')

    assert mocked_function.mock_calls == [mocker.call(
        ADOBE_OAUTH_TOKEN_URL,
        code='code_test',
        client_secret='appli_secret_test',
        authorization_response='/')]


def test_oauth_refresh_token(mocker):
    mocked_function = mocker.patch(
        "requests_oauthlib.OAuth2Session.refresh_token")
    AdobeSignOAuthSession.refresh_token('refresh_token_test', 'appli_id_test',
                                        'appli_secret_test')
    assert mocked_function.mock_calls == [mocker.call(
        ADOBE_OAUTH_REFRESH_TOKEN_URL,
        refresh_token='refresh_token_test',
        client_id='appli_id_test',
        client_secret='appli_secret_test',
        authorization_response='/')]


@pytest.fixture()
def adobe_sign_client():
    return AdobeSignClient(root_url='http://test',
                           access_token='TestToken',
                           api_user='test_api_user',
                           on_behalf_of_user='test_on_behalf_user')


@pytest.fixture()
def expected_headers():
    return {
        'Authorization': 'Bearer TestToken',
        'x-api-user': 'test_api_user',
        'x-on-behalf-of-user': 'test_on_behalf_user'
    }


@pytest.fixture()
def expected_participant():
    return {
        'name': 'Poney poney',
        'memberInfos': [
            {'email': 'poney@plop.com'}],
        'order': 1,
        'role': 'SIGNER'
    }


def test_adobe_api_version_should_be_v6(adobe_sign_client):
    assert adobe_sign_client.build_url(
        'mytest/resource') == 'http://test/api/rest/v6/mytest/resource'


def test_get_full_header(adobe_sign_client, expected_headers):
    headers = adobe_sign_client.get_headers()
    assert headers == expected_headers


def test_should_get_jsonified_participant(adobe_sign_client,
                                          expected_participant):
    jsonified_participant = adobe_sign_client.jsonify_participant(
        name='Poney poney',
        email='poney@plop.com',
        order=1)
    assert jsonified_participant == expected_participant


@pytest.fixture()
def test_document(mocker):
    document = mocker.Mock()
    document.name = '/tmp/test_document.pdf'
    document.bytes = b''
    return document


def test_call_upload_document(mocker, adobe_sign_client, expected_headers,
                              test_document):
    mocked_post = mocker.patch('requests.post')
    adobe_sign_client.upload_document(test_document)

    expected_data = {
        'File-Name': 'test_document.pdf',
        'Mime-Type': 'application/pdf'}
    mandatory_parameters = mocked_post.call_args[0]
    assert mandatory_parameters == (
        'http://test/api/rest/v6/transientDocuments',)

    kwargs_params = mocked_post.call_args[1]
    assert kwargs_params == {'headers': expected_headers,
                             'files': {'File': test_document.bytes},
                             'data': expected_data}


@pytest.fixture()
def response_with_error():
    def __get_response(error_code):
        response_with_client_error = Response()
        response_with_client_error.status_code = error_code
        return response_with_client_error

    return __get_response


@pytest.mark.parametrize('error_code', (404, 500))
def test_document_upload_client_or_server_error(error_code, mocker,
                                                response_with_error,
                                                adobe_sign_client,
                                                test_document):
    mocker.patch('requests.post', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.upload_document(test_document)


def test_should_create_signature(mocker, adobe_sign_client,
                                 expected_participant, expected_headers):
    mocked_post = mocker.patch('requests.post')
    participants = [expected_participant]
    adobe_sign_client \
        .post_agreement(transient_document_id='test_doc_id',
                        name='test_name',
                        participants=participants,
                        post_sign_redirect_url='http://post/sign/red/url',
                        post_sign_redirect_delay=12,
                        state='AUTHORING',
                        send_mail=False,
                        extra_param_str='plop',
                        extra_param_dict={'poney': 42},
                        extra_param_list=[1, 2, 3])
    expected_json = {
        'fileInfos': [{
            'transientDocumentId': 'test_doc_id'
        }],
        'name': 'test_name',
        'participantSetsInfo': [expected_participant],
        'signatureType': 'ESIGN',
        'state': 'AUTHORING',
        'postSignOption': {
            "redirectDelay": 12,
            "redirectUrl": 'http://post/sign/red/url'
        },
        'emailOption': {'sendOptions': {'completionEmails': 'NONE',
                                        'inFlightEmails': 'NONE',
                                        'initEmails': 'NONE'}
                        },
        'extra_param_str': 'plop',
        'extra_param_dict': {'poney': 42},
        'extra_param_list': [1, 2, 3]
    }
    mandatory_parameters = mocked_post.call_args[0]
    assert mandatory_parameters == ('http://test/api/rest/v6/agreements',)

    kwargs_params = mocked_post.call_args[1]
    assert kwargs_params == {'headers': expected_headers,
                             'json': expected_json}


@pytest.mark.parametrize('error_code', (404, 500))
def test_post_agreement_client_or_server_error(error_code, mocker,
                                               response_with_error,
                                               adobe_sign_client):
    expected_json_reply_error = {'code': str(error_code),
                                 'message': 'error raison'}
    response = response_with_error(error_code)
    mocker.patch.object(response, 'json',
                        return_value=expected_json_reply_error)
    mocker.patch('requests.post', return_value=response)
    with pytest.raises(AdobeSignException) as e:
        adobe_sign_client.post_agreement('doc id', 'name', [], '-', '-', False)
    assert expected_json_reply_error['code'] in str(e)
    assert expected_json_reply_error['message'] in str(e)


def test_get_agreements(mocker, adobe_sign_client, expected_headers):
    mocked_get = mocker.patch('requests.get')
    adobe_sign_client.get_agreements(page_size=12,
                                     cursor=44,
                                     extra_param_test={'test': 1})
    expected_params = {'pageSize': 12,
                       'cursor': 44,
                       'extra_param_test': {'test': 1}}
    mandatory_parameters = mocked_get.call_args[0]
    assert mandatory_parameters == ('http://test/api/rest/v6/agreements',)

    kwargs_params = mocked_get.call_args[1]
    assert kwargs_params == {'headers': expected_headers,
                             'params': expected_params}


@pytest.mark.parametrize('error_code', (404, 500))
def test_agreement_client_or_server_error(error_code, mocker,
                                          response_with_error,
                                          adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.get_agreements(11)


@pytest.mark.parametrize('include_next_participant_set', (True, False))
def test_get_members(include_next_participant_set, mocker, adobe_sign_client,
                     expected_headers):
    mocked_get = mocker.patch('requests.get')
    adobe_sign_client.get_members('test_agreement_id',
                                  include_next_participant_set)

    mandatory_parameters = mocked_get.call_args[0]
    assert mandatory_parameters == (
        'http://test/api/rest/v6/agreements/test_agreement_id/members',)

    kwargs_params = mocked_get.call_args[1]
    assert kwargs_params == {
        'params': {'includeNextParticipantSet': include_next_participant_set},
        'headers': expected_headers}


@pytest.mark.parametrize('error_code', (404, 500))
def test_get_members_client_or_server_error(error_code, mocker,
                                            response_with_error,
                                            adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.get_members('id', True)


def test_get_signing_url(mocker, adobe_sign_client, expected_headers):
    mocked_get = mocker.patch('requests.get')
    adobe_sign_client.get_signing_url('test_agreement_id')

    mandatory_parameters = mocked_get.call_args[0]
    assert mandatory_parameters == (
        'http://test/api/rest/v6/agreements/test_agreement_id/signingUrls',)

    kwargs_parameters = mocked_get.call_args[1]
    assert kwargs_parameters == {'headers': expected_headers}


@pytest.mark.parametrize('error_code', (404, 500))
def test_get_signing_url_client_or_server_error(error_code, mocker,
                                                response_with_error,
                                                adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.get_signing_url('id')


@pytest.mark.parametrize('code_reason', ('AGREEMENT_EXPIRED',
                                         'AGREEMENT_NOT_SIGNABLE',
                                         'AGREEMENT_NOT_VISIBLE'))
def test_get_signing_url_client_not_real_not_found_error(code_reason, mocker,
                                                         response_with_error,
                                                         adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(404))
    mocker.patch('requests.Response.json',
                 return_value={'code': code_reason, 'message': 'test'})
    with pytest.raises(AdobeSignNoMoreSignerException):
        adobe_sign_client.get_signing_url('id')


def test_get_signing_url_client_not_found_error(mocker,
                                                response_with_error,
                                                adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(404))
    mocker.patch('requests.Response.json',
                 return_value={'code': 'REAL_NOT_FOUND'})
    with pytest.raises(AdobeSignException) as excinfo:
        adobe_sign_client.get_signing_url('id')
    assert not excinfo.errisinstance(AdobeSignNoMoreSignerException)


def test_get_signer(mocker, adobe_sign_client, expected_headers):
    mocked_get = mocker.patch('requests.get')
    adobe_sign_client.get_signer('test_agreement_id', 'signer_id')

    mandatory_parameters = mocked_get.call_args[0]
    expected_url = 'http://test/api/rest/v6/agreements/test_agreement_id/' \
                   'members/participantSets/signer_id'
    assert mandatory_parameters == (expected_url,)

    kwargs_parameters = mocked_get.call_args[1]
    assert kwargs_parameters == {'headers': expected_headers}


@pytest.mark.parametrize('error_code', (404, 500))
def test_get_signer_client_or_server_error(error_code, mocker,
                                           response_with_error,
                                           adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.get_signer('test_agreement_id', 'signer_id')


def test_get_documents(mocker, adobe_sign_client, expected_headers):
    mocked_get = mocker.patch('requests.get')
    adobe_sign_client.get_documents('test_agreement_id', test_data={'a': 2})

    mandatory_parameters = mocked_get.call_args[0]
    assert mandatory_parameters == ('http://test/api/rest/v6/agreements/'
                                    'test_agreement_id/documents',)

    kwargs_parameters = mocked_get.call_args[1]
    assert kwargs_parameters == {'headers': expected_headers,
                                 'data': {'test_data': {'a': 2}}}


@pytest.mark.parametrize('error_code', (404, 500))
def test_get_documents_client_or_server_error(error_code, mocker,
                                              response_with_error,
                                              adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.get_documents('test_agreement_id')


def test_get_document(mocker, adobe_sign_client, expected_headers):
    mocked_get = mocker.patch('requests.get')
    adobe_sign_client.get_document('test_agreement_id', 'test_doc_id')

    mandatory_parameters = mocked_get.call_args[0]
    assert mandatory_parameters == ('http://test/api/rest/v6/agreements/'
                                    'test_agreement_id/documents/test_doc_id',)

    kwargs_parameters = mocked_get.call_args[1]
    assert kwargs_parameters == {'headers': expected_headers}


@pytest.mark.parametrize('error_code', (404, 500))
def test_get_document_client_or_server_error(error_code, mocker,
                                             response_with_error,
                                             adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(error_code))
    with pytest.raises(AdobeSignException):
        adobe_sign_client.get_document('test_agreement_id', 'test_doc_id')


def test_do_not_change_exception_if_not_invalid_token(mocker,
                                                      response_with_error,
                                                      adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(401))
    mocker.patch('requests.Response.json',
                 return_value={'code': 'other Reason',
                               'message': 'test'})
    with pytest.raises(AdobeSignException) as excinfo:
        adobe_sign_client.get_document('test_agreement_id', 'test_doc_id')
    assert not excinfo.errisinstance(AdobeSignInvalidAccessTokenException)


def test_raise_invalid_token_exception(mocker,
                                       response_with_error,
                                       adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(401))
    mocker.patch('requests.Response.json',
                 return_value={'code': 'INVALID_ACCESS_TOKEN',
                               'message': 'test'})
    with pytest.raises(AdobeSignInvalidAccessTokenException):
        adobe_sign_client.get_document('test_agreement_id', 'test_doc_id')


def test_raise_invalid_user_exception(mocker,
                                      response_with_error,
                                      adobe_sign_client):
    mocker.patch('requests.get', return_value=response_with_error(401))
    mocker.patch('requests.Response.json',
                 return_value={'code': 'INVALID_USER',
                               'message': 'test'})
    with pytest.raises(AdobeSignInvalidUserException):
        adobe_sign_client.get_document('test_agreement_id', 'test_doc_id')
